<?php

namespace EventPortal\Tests\Unit\Delegate;

use EventPortal\Domain\Delegate\DelegateForm;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Delegate\Models\DelegateFormStep;
use EventPortal\Domain\Event\Form\Enums\State;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class DelegateFormTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    public function is_true_when_all_steps_are_completed(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        factory(DelegateFormStep::class)->create([
            'state' => State::Complete()->getValue(),
            'delegate_id' => $delegate->id,
            'form_step_id' => $step->id,
        ]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertTrue($delegateForm->allStepsCompleted());
    }

    /** @test **/
    public function check_for_completed_steps_only_checks_current_form(): void
    {
        $formA = factory(Form::class)->create();
        $stepA = factory(FormStep::class)->create(['form_id' => $formA->id]);

        $formB = factory(Form::class)->create();
        $stepB = factory(FormStep::class)->create(['form_id' => $formB->id]);

        $delegate = factory(Delegate::class)->create();

        factory(DelegateFormStep::class)->create([
            'state' => State::Complete()->getValue(),
            'delegate_id' => $delegate->id,
            'form_step_id' => $stepA->id,
        ]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($formB);

        $this->assertFalse($delegateForm->allStepsCompleted());
    }

    /** @test */
    public function is_false_when_all_steps_are_partially_completed(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        factory(DelegateFormStep::class)->create([
            'state' => State::Partial()->getValue(),
            'delegate_id' => $delegate->id,
            'form_step_id' => $step->id,
        ]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertFalse($delegateForm->allStepsCompleted());
    }

    /** @test */
    public function is_false_when_delegate_form_steps_are_missing(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        factory(FormStep::class)->create(['form_id' => $form->id]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertFalse($delegateForm->allStepsCompleted());
    }

    /** @test */
    public function can_get_next_form_step(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 1]);
        $step2 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 2]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertTrue($delegateForm->nextStep($step1)->is($step2));
    }

    /** @test */
    public function returns_same_step_if_its_the_last_step_when_getting_next_form_step(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 1]);
        $step2 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 2]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertTrue($delegateForm->nextStep($step2)->is($step2));
    }

    /** @test */
    public function can_get_previous_form_step(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 1]);
        $step2 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 2]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertTrue($delegateForm->previousStep($step2)->is($step1));
    }

    /** @test */
    public function is_registering_when_all_steps_are_not_completed(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 1]);
        $step2 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 2]);
        factory(DelegateFormStep::class)->create([
            'state' => State::Complete()->getValue(),
            'delegate_id' => $delegate->id,
            'form_step_id' => $step1->id,
        ]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertEquals('registering', $delegateForm->getDelegateTransition());
    }

    /** @test */
    public function is_registered_when_all_steps_are_completed(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->create();
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 1]);
        factory(DelegateFormStep::class)->create([
            'state' => State::Complete()->getValue(),
            'delegate_id' => $delegate->id,
            'form_step_id' => $step1->id,
        ]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertEquals('registered', $delegateForm->getDelegateTransition());
    }

    /** @test */
    public function is_declining_when_all_steps_are_not_completed(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->state('decline')->create();
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 1]);
        $step2 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 2]);
        factory(DelegateFormStep::class)->create([
            'state' => State::Complete()->getValue(),
            'delegate_id' => $delegate->id,
            'form_step_id' => $step1->id,
        ]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertEquals('declining', $delegateForm->getDelegateTransition());
    }

    /** @test */
    public function is_declined_when_all_steps_are_completed(): void
    {
        $delegate = factory(Delegate::class)->create();
        $form = factory(Form::class)->state('decline')->create();
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id, 'order' => 1]);
        factory(DelegateFormStep::class)->create([
            'state' => State::Complete()->getValue(),
            'delegate_id' => $delegate->id,
            'form_step_id' => $step1->id,
        ]);

        $delegateForm = DelegateForm::make()->forDelegate($delegate)->forForm($form);

        $this->assertEquals('declined', $delegateForm->getDelegateTransition());
    }
}
