<?php

namespace EventPortal\Tests\Unit\Event\ValueObjects;

use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Event\ValueObjects\UnableToAttendSection;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class UnableToAttendSectionTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test **/
    public function get_unable_to_attend_section()
    {
        $event = factory(Event::class)->state('snapshot')->create();

        factory(Form::class)->states(['snapshot', 'decline'])->create(['event_id' => $event->id]);

        $eventDelegate = factory(EventDelegate::class)->create(['event_id' => $event->id, 'status' => 'invited']);

        $unableToAttendSection = UnableToAttendSection::make()->forEventDelegate($eventDelegate)->get();

        $this->assertMatchesJsonSnapshot($unableToAttendSection);
    }

    /** @test **/
    public function will_not_show_if_delegate_has_registered()
    {
        $event = factory(Event::class)->state('snapshot')->create();

        factory(Form::class)->states(['snapshot', 'decline'])->create(['event_id' => $event->id]);

        $eventDelegate = factory(EventDelegate::class)
            ->state('registered')
            ->create(['event_id' => $event->id]);

        $unableToAttendSection = UnableToAttendSection::make()->forEventDelegate($eventDelegate);

        $this->assertFalse($unableToAttendSection->shouldShow());
    }

    /** @test **/
    public function will_not_show_if_delegate_has_these_states()
    {
        $states = [
            'candidate',
            'registered',
            'declined',
            'attended',
            'cancelled',
            'no_show',
        ];

        foreach ($states as $state) {
            $this->assertThatSectionShouldNotBeShown($state);
        }
    }

    /** @test **/
    public function will_show_if_delegate_has_these_states()
    {
        $states = [
            'invited',
            'registering',
            'declining',
        ];

        foreach ($states as $state) {
            $this->assertThatSectionShouldBeShown($state);
        }
    }

    private function assertThatSectionShouldNotBeShown(string $state)
    {
        $event = factory(Event::class)->state('snapshot')->create();

        factory(Form::class)->states(['snapshot', 'decline'])->create(['event_id' => $event->id]);

        $eventDelegate = factory(EventDelegate::class)
            ->state($state)
            ->create(['event_id' => $event->id]);

        $unableToAttendSection = UnableToAttendSection::make()->forEventDelegate($eventDelegate);

        $this->assertFalse($unableToAttendSection->shouldShow());
    }

    private function assertThatSectionShouldBeShown(string $state)
    {
        $event = factory(Event::class)->state('snapshot')->create();

        factory(Form::class)->states(['snapshot', 'decline'])->create(['event_id' => $event->id]);

        $eventDelegate = factory(EventDelegate::class)
            ->state($state)
            ->create(['event_id' => $event->id]);

        $unableToAttendSection = UnableToAttendSection::make()->forEventDelegate($eventDelegate);

        $this->assertTrue($unableToAttendSection->shouldShow());
    }
}
